/* 
   Draw functions
   --------------------------------------------------------------------
   Linberto

   Copyright (C) 1999-2001 Diego Javier Grigna <diego@grigna.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"
#include "draw.h"

void draw_berto_board( int ndx)
{
 int i, j;

 for( i = 0; i < 7; i++)
      for( j = 0; j < 15; j++)
           if( map_stage[ i][ j] != -1)
               draw_block( BLOCK_X + (j * 16), BLOCK_Y + (i * 21), lcolors[ ndx]);

}

void draw_block( int x, int y, struct level_colors lc)
{
 switch( opt.cfg_blockkind) {
         case BLOCK_PLAIN:   draw_cube( x, y, lc);
                             break;
         case BLOCK_TEXTURE: draw_texture_cube( x, y, lc);
                             break;
         case BLOCK_WIRE:    draw_wire_cube( x, y, lc);
                             break;
         case BLOCK_SPHERE:  draw_sphere( x, y, lc);
                             break;
 }
}

void draw_cube( int x, int y, struct level_colors lc)
{
 draw_board_top(       x, y, lc.ctop);
 draw_board_sideleft(  x, y, lc.cleft);
 draw_board_sideright( x, y, lc.cright);
}

void draw_board_top( int sx, int sy, int sidecolor)
{
 int i, x, y;

 i = 0;
 x = sx;
 y = sy;

 while( y < sy + 8) {
        gl_line( x-1-i, y, x+i, y, sidecolor);
        i += 2;
        y++;
 }

 i-=2;

 while( y <= sy + 15) {
        gl_line( x-1-i, y, x+i, y, sidecolor);
        i -= 2;
        y++;
 }

}

void draw_board_sideleft( int sx, int sy, int sidecolor)
{
 int x, y;

 x = sx - 16;
 y = sy +  8;

 gl_line( x, y, x, y + 12, sidecolor);

 y++;
 x++;

 while( y < sy + 16) {
        gl_line( x    , y, x    , y + 12, sidecolor);
        gl_line( x + 1, y, x + 1, y + 12, sidecolor);
        y++;
        x+=2;
 }

 gl_line( x, y, x, y + 12, sidecolor);
}

void draw_board_sideright( int sx, int sy, int sidecolor)
{
 int x, y;

 x = sx + 15;
 y = sy +  8;

 gl_line( x, y, x, y + 12, sidecolor);

 y++;
 x-=2;

 while( y < sy + 16) {
        gl_line( x    , y, x    , y + 12, sidecolor);
        gl_line( x + 1, y, x + 1, y + 12, sidecolor);
        y++;
        x-=2;
 }
 x++;
 gl_line( x, y, x, y + 12, sidecolor);
}

void draw_mini_board( int x, int y)
{
 int i, j;

 for( i = 0; i < 7; i++)
      for( j = 0; j < 15; j++)
           if( map_stage[ i][ j] != -1)
               draw_mini_cube( x + (j * 8), y + (i * 10), lcolors[ 0]);

}

void draw_mini_cube( int x, int y, struct level_colors lc)
{
 draw_mini_board_top(       x, y, lc.ctop);
 draw_mini_board_sideleft(  x, y, lc.cleft);
 draw_mini_board_sideright( x, y, lc.cright);
}

void draw_mini_board_top( int sx, int sy, int sidecolor)
{
 int i, x, y;

 i = 0;
 x = sx;
 y = sy;

 while( y < sy + 4) {
        gl_line( x-1-i, y, x+i, y, sidecolor);
        i += 2;
        y++;
 }

 i-=2;

 while( y <= sy + 7) {
        gl_line( x-1-i, y, x+i, y, sidecolor);
        i -= 2;
        y++;
 }
}

void draw_mini_board_sideleft( int sx, int sy, int sidecolor)
{
 int x, y;

 x = sx - 8;
 y = sy + 4;

 gl_line( x, y, x, y + 6, sidecolor);

 y++;
 x++;

 while( y < sy + 8) {
        gl_line( x    , y, x    , y + 6, sidecolor);
        gl_line( x + 1, y, x + 1, y + 6, sidecolor);
        y++;
        x+=2;
 }

 gl_line( x, y, x, y + 6, sidecolor);
}

void draw_mini_board_sideright( int sx, int sy, int sidecolor)
{
 int x, y;

 x = sx + 7;
 y = sy + 4;

 gl_line( x, y, x, y + 6, sidecolor);

 y++;
 x-=2;

 while( y < sy + 8) {
        gl_line( x    , y, x    , y + 6, sidecolor);
        gl_line( x + 1, y, x + 1, y + 6, sidecolor);
        y++;
        x-=2;
 }
 x++;
 gl_line( x, y, x, y + 6, sidecolor);
}

void draw_wire_cube( int x, int y, struct level_colors lc)
{
 gl_line( x -  1, y + 12, x - 16, y + 20, lc.cright);
 gl_line( x     , y + 12, x + 15, y + 20, lc.cright);
 gl_line( x - 16, y + 20, x     , y + 28, lc.cleft );
 gl_line( x     , y + 28, x + 15, y + 20, lc.cleft );

 gl_line( x -  1, y     , x - 16, y +  8, lc.cleft );
 gl_line( x     , y     , x + 15, y +  8, lc.cleft );
 gl_line( x - 16, y +  8, x     , y + 16, lc.cleft );
 gl_line( x     , y + 16, x + 15, y +  8, lc.cleft );

 gl_line( x -  1, y +  1, x -  1, y + 12, lc.cright);
 gl_line( x     , y +  1, x     , y + 12, lc.cright);
 gl_line( x - 16, y +  8, x - 16, y + 20, lc.cleft );

 gl_line( x + 15, y +  8, x + 15, y + 20, lc.cleft );
 gl_line( x -  1, y + 16, x -  1, y + 28, lc.cleft );
 gl_line( x     , y + 16, x     , y + 28, lc.cleft );
}

void draw_texture_cube( int x, int y, struct level_colors lc)
{
 draw_texture_top(       x, y, lc.ctop);
 draw_texture_sideleft(  x, y, lc.cleft);
 draw_texture_sideright( x, y, lc.cright);
}

void draw_texture_top( int sx, int sy, int sidecolor)
{
 int i, x, y, c;

 if( sidecolor == WHITE_TOP)
     c = WHITE_TOP - 4;
 else 
     c = sidecolor;

 i = 0;
 x = sx;
 y = sy;

 while( y < sy + 8) {
        gl_line( x-1-i, y, x+i, y, c + y % 4);
        i += 2;
        y++;
 }

 i-=2;

 while( y <= sy + 15) {
        gl_line( x-1-i, y, x+i, y, c + y % 4);
        i -= 2;
        y++;
 }
}

void draw_texture_sideleft( int sx, int sy, int sidecolor)
{
 int x, y;

 x = sx - 16;
 y = sy +  8;

 gl_line( x, y, x, y + 12, sidecolor - y % 4);

 y++;
 x++;

 while( y < sy + 16) {
        gl_line( x    , y, x    , y + 12, sidecolor - y % 4);
        gl_line( x + 1, y, x + 1, y + 12, sidecolor - y % 4);
        y++;
        x+=2;
 }

 gl_line( x, y, x, y + 12, sidecolor - y % 4);
}

void draw_texture_sideright( int sx, int sy, int sidecolor)
{
 int x, y;

 x = sx + 15;
 y = sy +  8;

 gl_line( x, y, x, y + 12, sidecolor + y % 4);

 y++;
 x-=2;

 while( y < sy + 16) {
        gl_line( x    , y, x    , y + 12, sidecolor + y % 4);
        gl_line( x + 1, y, x + 1, y + 12, sidecolor + y % 4);
        y++;
        x-=2;
 }
 x++;
 gl_line( x, y, x, y + 12, sidecolor + y % 4);
}

void draw_bar( int x, int y, int len, int pos, int col)
{
 int lpos;

 lpos = pos * menu_spacing;

 draw_rectangle( x, y + lpos, len, 12, col);
}

void draw_sphere( int x, int y, struct level_colors lc)
{
 int radius;

 for( radius = 0; radius < 14; radius++)
      draw_circle( x, y + 12, radius, lc.cleft - radius);
}


/*
 * Yes, I know that SVGALIB have the gl_circle() function,
 * but I want to use my own.
 */

void draw_circle( int x, int y, int r, int c)
{
 int angle;
 int x2, y2;

 for( angle = 0; angle < 360; angle++) {
      x2 = x + cos_table[ angle] * r;
      y2 = y + sin_table[ angle] * r;
      if( x2 < 319 && x2 > 0)
          if( y2 < 199 && y2 > 0)
              gl_setpixel( x2, y2, c);
 }

}

void draw_rectangle( int x, int y, int w, int h, int c)
{
 gl_fillbox( x        , y        , 2, h, c);
 gl_fillbox( x + w - 2, y        , 2, h, c);
 gl_fillbox( x        , y        , w, 2, c);
 gl_fillbox( x        , y + h - 2, w, 2, c);
}

