/*
   Score functions
   --------------------------------------------------------------------
   Linberto

   Copyright (C) 1999-2001 Diego Javier Grigna <diego@grigna.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"

static void score_enter_name( FILE *fp, long record);
static void score_draw_bar( int pos, char *name);
static void score_move_score( FILE *fp, struct score_struct ss, long record, long maxrecord);

void score_show( void)
{
 FILE *fp;
 struct stat stbuf;
 struct score_struct ss;
 int y;

 check_if_file_exist( LINBERTO_SCOREFILE);

 fp = linberto_openfile( LINBERTO_SCOREFILE, LINBERTO_SCOREFILE_DEFAULT, "r");

 fstat( fileno( fp), &stbuf);

 gl_clearscreen( 0);
 gl_write( 0,  0, _("  Name            Score"));
 lin_title_line();

 y = 20;

 if( stbuf.st_size == 0) {
     for( y = 20; y < 180; y += 10)
          gl_write( 0,  y, "  ---             0");
 } else {
     while( !feof( fp) && fread( &ss, sizeof( struct score_struct), 1, fp) > 0 && y < 180) {
            grf_printf( 0,  y, "  %-3.3s             %ld", ss.name, ss.score);
            y += 10;
     }
 }

 fclose( fp);

 for( ; y < 180; y += 10)
      gl_write( 0,  y, "  ---             0");

 press_enter_str();

 key_wait_for_enter();
}

void score_high_check( void)
{
 FILE *fp;
 struct stat stbuf;
 struct score_struct ss;
 long maxrecord;
 long count;

 if( score <= 0)
     return;

 if( opt.cfg_invincible == INVINCIBLE_ON)
     return;

 check_if_file_exist( LINBERTO_SCOREFILE);

 fp = linberto_openfile( LINBERTO_SCOREFILE, LINBERTO_SCOREFILE_DEFAULT, "r+");

 fstat( fileno( fp), &stbuf);

 maxrecord = stbuf.st_size / sizeof( struct score_struct);

 if( maxrecord > 15)
     maxrecord = 15;

 maxrecord--;

 count = 0;

 while( !feof( fp) && fread( &ss, sizeof( struct score_struct), 1, fp) > 0 && count < 16) {
        if( score > ss.score) {
            score_enter_name( fp, count);
            score_move_score( fp, ss, count, maxrecord);
            count = 16;
            break;
        }
        count++;
 }

 if( count < 16)
     score_enter_name( fp, count);

 fclose( fp);
}

static void score_enter_name( FILE *fp, long record)
{
 struct score_struct ss;
 int pos;
 
 gl_clearscreen( 0);

 ss.name[ 0] = ss.name[ 1] = ss.name[ 2] = '-';
 ss.name[ 3] = 0;

 pos = 0;

 score_draw_bar( pos, ss.name);

 while( 1) {
        if( keyboard_update() == 0 && joy_moved() == 0)
            continue;

        if( pos < 0) pos = 3;
        if( pos > 3) pos = 0;

        score_draw_bar( pos, ss.name);

        switch( lin_scancode) {
           case SCANCODE_ENTER:
           case SCANCODE_KEYPADENTER:
                  if( pos == 3)
                      goto outwhile;
                  else
                      pos++;
                  continue;
           case SCANCODE_ESCAPE:
                  goto outwhile;
           case SCANCODE_CURSORUP:
           case SCANCODE_CURSORBLOCKUP:
                  if( pos != 3) {
                      ss.name[ pos] = key_find_char( ss.name[ pos], -1);
                  } else {
                      pos--;
                  }
                  continue;
           case SCANCODE_CURSORLEFT:
           case SCANCODE_CURSORBLOCKLEFT:
           case SCANCODE_BACKSPACE:
                  sound_send( SND_MENU);
                  pos--;
                  continue;
           case SCANCODE_CURSORDOWN:
           case SCANCODE_CURSORBLOCKDOWN:
                  if( pos != 3) {
                      ss.name[ pos] = key_find_char( ss.name[ pos], 1);
                  } else {
                      pos++;
                  }
                  continue;
           case SCANCODE_CURSORRIGHT:
           case SCANCODE_CURSORBLOCKRIGHT:
           case SCANCODE_TAB:
                  sound_send( SND_MENU);
                  pos++;
                  continue;
           case SCANCODE_HOME:
           case SCANCODE_KEYPAD7:
                  sound_send( SND_MENU);
                  pos = 0;
                  continue;
           case SCANCODE_END:
           case SCANCODE_KEYPAD1:
                  sound_send( SND_MENU);
                  pos = 3;
                  continue;
           case LIN_KEY_NONE:
                  continue;
           default:
                  ss.name[ pos] = keyttbl[ lin_scancode];
                  pos++;
                  continue;
        } /* end switch( lin_scancode) */

 } /* end while( 1) */

outwhile:

 ss.score = score;

 fseek( fp, ( long) record * sizeof( struct score_struct), SEEK_SET);
 fwrite( &ss, sizeof( struct score_struct), 1, fp);
}

static void score_draw_bar( int pos, char *name)
{
 int lpos;

 gl_write(  0,  20, _("         Enter your initials          "));
 grf_printf( 0,  40, "               %c  %c  %c                "
                   , name[ 0], name[ 1], name[ 2]);
 gl_write(  0,  60, _("                 END                  "));
 
 lpos = pos * 24;

 gl_fillbox( 118, 38, 12, 2, LIN_BLACK);
 gl_fillbox( 118, 48, 12, 2, LIN_BLACK);
 gl_fillbox( 118, 40,  2, 8, LIN_BLACK);
 gl_fillbox( 128, 40,  2, 8, LIN_BLACK);

 gl_fillbox( 142, 38, 12, 2, LIN_BLACK);
 gl_fillbox( 142, 48, 12, 2, LIN_BLACK);
 gl_fillbox( 142, 40,  2, 8, LIN_BLACK);
 gl_fillbox( 152, 40,  2, 8, LIN_BLACK);

 gl_fillbox( 166, 38, 12, 2, LIN_BLACK);
 gl_fillbox( 166, 48, 12, 2, LIN_BLACK);
 gl_fillbox( 166, 40,  2, 8, LIN_BLACK);
 gl_fillbox( 176, 40,  2, 8, LIN_BLACK);

 gl_fillbox( 126, 58, 44, 2, LIN_BLACK);
 gl_fillbox( 126, 68, 44, 2, LIN_BLACK);
 gl_fillbox( 126, 60,  2, 8, LIN_BLACK);
 gl_fillbox( 168, 60,  2, 8, LIN_BLACK);

 if( pos != 3) {
     gl_fillbox( 118 + lpos, 38, 12, 2, LIN_RED);
     gl_fillbox( 118 + lpos, 48, 12, 2, LIN_RED);
     gl_fillbox( 118 + lpos, 40,  2, 8, LIN_RED);
     gl_fillbox( 128 + lpos, 40,  2, 8, LIN_RED);
 }
 else {
     gl_fillbox( 126, 58       , 44, 2, LIN_RED);
     gl_fillbox( 126, 68       , 44, 2, LIN_RED);
     gl_fillbox( 126, 60       ,  2, 8, LIN_RED);
     gl_fillbox( 168, 60       ,  2, 8, LIN_RED);
 }
}

static void score_move_score( FILE *fp, struct score_struct ss, 
                                        long record, long maxrecord)
{
 struct score_struct temp;

 if( record > maxrecord)
     return;

 /* Go to the next record */
 record++;

 /* Read the score        */
 fseek( fp, ( long) record * sizeof( struct score_struct), SEEK_SET);
 fread( &temp, sizeof( struct score_struct), 1, fp);

 /* Write the new score   */
 fseek( fp, ( long) record * sizeof( struct score_struct), SEEK_SET);
 fwrite( &ss, sizeof( struct score_struct), 1, fp);

 /* Move the next scores */
 score_move_score( fp, temp, record, maxrecord);
}

